-- main steamctl process
-- performs steam controlling function linked to mqtt plus launches steam status reporting script

local cfg = require("kibo:cfg")
local lua = require("kibo:lua")
local exec = require("kibo:exec")
local mqtt = require("kibo:mqtt")
local log = require("kibo:log")("steamctl.control")

-- determine local host name
local host = exec("hostname"):stdout():gsub("\n", "")
-- define mqtt client
local mqtt_uri = cfg.getstr("flespi-mqtt-uri")
local mqtt_token = cfg.getstr("control-flespi-token")

local mqtt_client = mqtt:create()
mqtt_client:set_ssl_certificates(mqtt_uri)

-- conect to MQTT broker and subscribe to control topic
local function mqtt_connect()
	-- connect and subscribe
	if mqtt_client:connect({uri = mqtt_uri, username = mqtt_token, client_id = "steamctl-"..host, clean_session = true, expiry_interval=0, version=5}) ~= true then
		log:warn("mqtt_client:connect failed to uri='%s'", mqtt_uri)
		error("mqtt_client:connect failed to uri "..mqtt_uri)
	else
		log:info("connected to uri='%s'", mqtt_uri)
	end
	-- if connected then subscribe to host control topic with qos=1
	mqtt_client:subscribe("steamctl/control/"..host, 1)
end

-- call for mqtt connect with error exit on failure
mqtt_connect()

-- launch periodic status reporter
local status = require(cfg.getstr("root").."/status.lua")
lua.schedule(function()
	while true do
		local ok, err = pcall(function()
			status:report()
		end)
		if not ok then
			log:warn(err)
		end
		sleep(60)
	end
end)

-- launch controller
lua.schedule(function()
	while true do
		local ok, err = pcall(function()
			status:control()
		end)
		if not ok then
			log:warn(err)
		end
		sleep(10)
	end
end)

-- run forever normal working cycle forever
while true do
	-- check for new messages for MQTT client
	local ok, err = pcall(function()
		-- perform connection check
		if not mqtt_client:connected() then
			mqtt_connect()
		end
		-- wait for new messages (or wait for connection close)
		local msg = mqtt_client:wait()
		while msg ~= false do
			-- ack message first
			mqtt_client:ack(msg.packet_id)
			-- now operate with the message
			log:print("received message with topic='%s' and payload='%s'", msg.topic, msg.payload)
			-- based on the message payload either enable or disable steam
			if msg.payload == "true" then
				-- enable steam
				status.enable(true)
			else
				-- in any other case disable it
				status.enable(false)
			end
			msg = mqtt_client:wait()
		end
	end)
	-- processing done
	if not ok then
		log:warn(err)
	end
	sleep(10)
end
